@@ -6,13 +6,10 @@ use apexl\Io\includes\Controller;
 use apexl\Io\includes\Hook;
 use apexl\Io\includes\Routes;
 use apexl\Io\includes\System;
-use apexl\Io\modules\company\entities\companyEntity;
 use apexl\Io\modules\display\components\BasicLink;
 use apexl\Io\modules\display\components\ButtonBar;
-use apexl\Io\modules\display\components\CardEntityFilteredDisplayTable;
 use apexl\Io\modules\display\components\ColWrapper;
 use apexl\Io\modules\display\components\ContentTitle;
-use apexl\Io\modules\display\components\DisplayTable;
 use apexl\Io\modules\display\components\EntityFilteredDisplayTable;
 use apexl\Io\modules\display\components\FormComponent;
 use apexl\Io\modules\display\components\genericComponents;
@@ -21,19 +18,16 @@ use apexl\Io\modules\display\services\Render;
 use apexl\Io\modules\formbuilder\entities\formEntity;
 use apexl\Io\modules\formbuilder\includes\checkboxField;
 use apexl\Io\modules\formbuilder\includes\inputField;
-use apexl\Io\modules\formbuilder\includes\markupField;
-use apexl\Io\modules\formbuilder\includes\selectField;
-use apexl\Io\modules\subscription\entities\subscriptionEntity;
-use apexl\Io\modules\subscriptionDisplay\components\viewTiles;
+use apexl\Io\modules\user\entities\roleEntity;
+use apexl\Io\modules\user\entities\userEntity;
+use apexl\Io\modules\user\enums\permissions\Role;
+use apexl\Io\modules\user\enums\permissions\User;
+use apexl\Io\modules\user\services\currentUser;
 use apexl\Io\modules\userDisplay\components\dashboardTiles;
 use apexl\Io\modules\userDisplay\components\roleEntityDisplayTile;
 use apexl\Io\modules\userDisplay\components\userEntityDisplayTile;
 use apexl\Io\modules\userDisplay\forms\forgottenPasswordForm;
-use apexl\Io\modules\userDisplay\forms\loginAsUserForm;
 use apexl\Io\modules\userDisplay\forms\loginForm;
-use apexl\Io\modules\user\entities\roleEntity;
-use apexl\Io\modules\user\entities\userEntity;
-use apexl\Io\modules\user\services\currentUser;
 use apexl\Io\services\Output;
 use Psr\Http\Message\ResponseInterface as Response;
 use Psr\Http\Message\ServerRequestInterface as Request;
@@ -51,24 +45,17 @@ class userController extends Controller
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @param $args
      * @return Response
      * @throws \Exception
      */
     public function userViewSingle(Request $request, Response $response, $args)
     {
-
-        $userEntity = userEntity::load($args['id']);
-
         $rowWrapper = (new RowWrapper())->addClass('manageForm-wrapper');
-        $rowWrapper->addComponent(dashboardTiles::userDetailsTile($args['id']));
-
-        $rowWrapper->addComponent(\apexl\Io\modules\invoiceDisplay\components\dashboardTiles::userInvoicesTile($userEntity));
-        $rowWrapper->addComponent(viewTiles::userSubscriptionsTile($userEntity));
-
+        $rowWrapper->addComponent(dashboardTiles::userViewTile($args['id']));
 
+        $userEntity = new userEntity();
+        $userEntity->load($args['id']);
 
         $buttonsBar = new ButtonBar();
         $buttonsBar->setEntity($userEntity);
@@ -79,8 +66,6 @@ class userController extends Controller
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
     public function userSettingsPage(Request $request, Response $response)
@@ -95,8 +80,6 @@ class userController extends Controller
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      * @throws \Exception
      */
@@ -106,29 +89,144 @@ class userController extends Controller
     }
 
     /**
-     * Login page - @see userDisplayController in io-user
-     * @param Request $request
-     * @param Response $response
+     * @param  array  $args
      * @return Response
+     * @throws \Exception
+     */
+    public function userEditForm(Request $request, Response $response, array $args = [])
+    {
+        $form = $this->primaryAccountForm();
+        $form->setSubmitText('Submit');
+
+        //set form field values
+        $user = new userEntity();
+        if (isset($args[$user->primaryKey])) {
+            $user->load($args[$user->primaryKey]);
+            $form->setMethod('PUT');
+        }
+
+        $form->setActionUrl(Routes::getRoutePattern('userEntity.put'));
+
+        if ($this->currentUser->isAllowed(User::CHANGE_ACTIVE_STATE)) {
+            $activeField = (new checkboxField('active'))->setLabel('Active')->setValue($user->active);
+        } else {
+            $activeField = (new inputField('active'))->setInputType('hidden');
+        }
+
+        if ($id = $args[$user->primaryKey] ?? false) {
+            $form->setActionUrl(Routes::getRoutePattern($user->getEntityName().'.put', ['id' => $id]));
+
+            $form->addField($form->getFormField('first_name')->setValue($user->first_name));
+            $form->addField($form->getFormField('last_name')->setValue($user->last_name));
+            $form->addField($form->getFormField('email')->setValue($user->email));
+
+            $activeField->setChecked($user->active);
+        }
+
+        $form->addField($activeField);
+
+        $form->setFieldsetSetting('roles', 'fieldset_legend', 'Roles');
+        $roles = (new roleEntity())->loadMultiple();
+        foreach ($roles as $role) {
+            //for users that can't change roles, show labels for those that they do have.
+            $showLabel = false;
+            if ($this->currentUser->isAllowed(Role::UPDATE)) {
+                $roleField = (new checkboxField('role_'.$role->id))
+                    ->setLabel($role->name)
+                    ->setChecked($user->hasRole($role))
+                    ->setValue($user->hasRole($role));
+            } else {
+                $showLabel = true;
+                $roleField = (new inputField('role_'.$role->id));
+                $roleField->setInputType('hidden');
+            }
+            if (($user->id ?? false) && $user->hasRole($role)) {
+                //allows us to show the label only for those without the role to change.
+                //Only shows labels for the roles they have.
+                if ($showLabel) {
+                    $roleField->setLabel($role->name);
+                }
+            }
+            $form->addField($roleField, 'roles');
+        }
+
+        //allow other modules to modify form.
+        $form = Hook::processHook('displayFormRender', $form, $user, 'userEditForm');
+
+        //set role values
+        $this->output::addResponse($request, $form->getBuiltFormArray());
+
+        return System::asJson($response);
+    }
+
+    /**
+     * @param  bool  $passwordRequired
+     */
+    protected function primaryAccountForm($passwordRequired = false): formEntity
+    {
+        $form = new formEntity();
+        $form->setId('login');
+        $form->setMethod('post');
+        $form->setActionUrl(Routes::getRoutePattern('userEntity.post'));
+
+        $form->addField(
+            (new inputField('first_name'))->setLabel('First Name')->setPlaceholder('First Name')->isRequired()
+        );
+        $form->addField((new inputField('last_name'))->setLabel('Surname')->setPlaceholder('Surname')->isRequired());
+        $form->addField(
+            (new inputField('email'))->setLabel('Email')->setPlaceholder('Email')->setInputType('email')->isRequired()
+        );
+
+        $password = (new inputField('password'))->setLabel('Password')->setPlaceholder('Password')->setInputType(
+            'password'
+        )->addValidation("minLength", ["min" => 6]);
+        $confirmPass = (new inputField('confirm_password'))->setLabel('Confirm Password')->setPlaceholder(
+            'Confirm Password'
+        )->setInputType('password')->addValidation("minLength", ["min" => 6]);
+        if ($passwordRequired) {
+            $password->isRequired();
+            $confirmPass->isRequired();
+        }
+        $form->addField($password);
+        $form->addField($confirmPass);
+
+        return $form;
+    }
+
+    /**
+     * Login page - @return Response
+     * @see userDisplayController in io-user
      */
     public function loginPage(Request $request, Response $response, $args)
     {
         if (currentUser::getCurrentUser()->isLoggedIn()) {
             Output::addMetadata('user.login.redirect', 'redirect', '/');
+
             return System::asJson($response);
         }
 
         $this->render::setActive("noUI");
-        $this->render::setMetaTitle("Login | ". ($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
+        $this->render::setMetaTitle("Login | ".($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
         $components = [];
         if (isset($args['loginMessageType']) && isset($this->config->app->loginMessageTypes->{$args['loginMessageType']})) {
-            $components[] = (new ContentTitle())->addContent($this->config->app->loginMessageTypes->{$args['loginMessageType']})->addClass('text-light');
+            $components[] = (new ContentTitle())->addContent(
+                $this->config->app->loginMessageTypes->{$args['loginMessageType']}
+            )->addClass('text-light');
         }
-        $components[] = (new FormComponent())->src(Routes::getRoutePattern('user.display.forms.login'))
+        $redirect = $request->getQueryParams()['redirect'] ?? null;
+        $redirect = $redirect ? sprintf('?redirect=%s', htmlentities($redirect)) : null;
+        $components[] = (new FormComponent())->src(
+            Routes::getRoutePattern('user.display.forms.login').$redirect
+        )
             ->title(System::getVariable('user_login_description_text') ?? "Sign in with your email address.")
             ->addClass('login-form')->setID('login');
-        $components[] = (new BasicLink())->addClass('forgottenPasswordLink')->addRoute(Routes::getRoutePattern('user.display.forgot-password'))->addText(System::getVariable('user_login_forgot_password_text') ?? 'Forgot password?');
-        $components[] = (new BasicLink())->addClass('registerLink')->addRoute(Routes::getRoutePattern('user.display.register'))->addText(System::getVariable('user_login_register_text') ?? 'No account? Register.');
+        $components[] = (new BasicLink())->addClass('forgottenPasswordLink')->addRoute(
+            Routes::getRoutePattern('user.display.forgot-password')
+        )->addText(System::getVariable('user_login_forgot_password_text') ?? 'Forgot password?');
+        $components[] = (new BasicLink())->addClass('registerLink')->addRoute(
+            Routes::getRoutePattern('user.display.register')
+        )->addText(System::getVariable('user_login_register_text') ?? 'No account? Register.');
+
         return System::asJson($response, $this->render::build($components));
     }
 
@@ -136,11 +234,12 @@ class userController extends Controller
     {
         if (currentUser::getCurrentUser()->isLoggedIn()) {
             Output::addMetadata('user.login.redirect', 'redirect', '/');
+
             return System::asJson($response);
         }
 
         $this->render::setActive("noUI");
-        $this->render::setMetaTitle("Register | ". ($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
+        $this->render::setMetaTitle("Register | ".($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
         $components = [];
 
         $components[] = (new FormComponent())->src(Routes::getRoutePattern('user.display.forms.register'))
@@ -155,61 +254,15 @@ class userController extends Controller
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
-    public function loginForm(Request $request, Response $response, $args)
+    public function loginForm(Request $request, Response $response)
     {
-        return System::asJson($response, (new loginForm())->loginForm($args));
-    }
-
-    public function loginAsUserForm(Request $request, Response $response, $args){
-        return System::asJson($response, (new loginAsUserForm())->loginAsUserForm($args));
-    }
-
-    public function manageSettingsForm(Request $request, Response $response, $args)
-    {
-        $currentUser = currentUser::getCurrentUser();
-        $form = new formEntity();
-        $form->setId('user-manage-settings-form');
-        $form->setSubmitText('Update Now');
-        $form->setMethod('post');
-        $form->setActionUrl(Routes::getRoutePattern('user.updateSettings'));
-        $settings = $currentUser->settings ?? null;
-        $minOptions = [
-            0 => 'Disabled',
-            5 => '5 mins',
-            10 => '10 mins',
-            20 => '20 mins',
-            20 => '30 mins',
-            60 => '60 mins'
-        ];
-        $form->addField((new markupField("systemSettingsTitle"))->setValue("<h4>System Settings</h4>"));
-        $form->addField((new selectField('logoutAfterInactivity', $minOptions))->setLabel('Logout after inactivity')->setValue($settings->logoutAfterInactivity ?? 0));
-        $form->addField((new selectField('refreshScreenContentEvery', $minOptions))->setLabel('Refresh Screen Content Every')->setValue($settings->refreshScreenContentEvery ?? 0));
-        $form->addField((new checkboxField('displayNotificationsOnScreen'))->setLabel('Display notifications on screen')->setValue($settings->displayNotificationsOnScreen ?? 0));
-        $form->addField((new markupField("cookieSettings"))->setValue("<h4>Cookie Settings</h4>"));
-        $form->addField((new checkboxField('allowTrackingCookies'))->setLabel('Allow tracking cookies')->setValue($settings->allowTrackingCookies ?? 1));
-        $form->addField((new markupField("usefulLinks"))->setValue("<h4>Useful Links</h4>"));
-        if ($href = System::getVariable('settings_contact_support_href')) {
-            $form->addField((new markupField("usefulLinks_contactSupport"))->setValue('<i class="fa-regular fa-circle-question"></i> <a target="_blank" href="'.$href.'">Contact Support</a>'));
-        }
-        if ($href = System::getVariable('settings_report_a_bug_href')) {
-            $form->addField((new markupField("usefulLinks_reportABug"))->setValue('<i class="fa-light fa-bug"></i> <a target="_blank" href="'.$href.'">Report a Bug</a>'));
-        }
-        if ($href = System::getVariable('settings_visit_website_href')) {
-            $form->addField((new markupField("usefulLinks_visitWebsite"))->setValue('<i class="fa-light fa-arrow-right"></i> <a target="_blank" href="'.$href.'">Visit Website</a>'));
-        }
-
-        return System::asJson($response, $form->getBuiltFormArray());
-
+        return System::asJson($response, (new loginForm())->loginForm($request));
     }
 
     /**
      * Forgotten Password Page
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
     public function forgotPasswordPage(Request $request, Response $response)
@@ -220,16 +273,18 @@ class userController extends Controller
             return System::asJson($response);
         }
         $this->render::setActive("noUI");
-        $this->render::setMetaTitle("Forgotten Password | ". ($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
+        $this->render::setMetaTitle("Forgotten Password | ".($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
 
         $rowWrapper = (new RowWrapper())->addClass('d-block');
         $rowWrapper->addComponent(
             (new FormComponent())->src(Routes::getRoutePattern('user.display.forms.forgottenPassword'))
-            ->title("Enter your email to reset your password.")
+                ->title("Enter your email to reset your password.")
                 ->addClass('forgotten-password-form')->setID('forgottenPassword')
         );
         $rowWrapper->addComponent(
-            (new BasicLink())->addClass('forgottenPasswordLink return mt-5 text-decoration-underline')->addRoute(Routes::getRoutePattern('user.display.login'))->addText(System::getVariable('user_login_return_login_text') ?? 'Return to login')
+            (new BasicLink())->addClass('forgottenPasswordLink return mt-5 text-decoration-underline')->addRoute(
+                Routes::getRoutePattern('user.display.login')
+            )->addText(System::getVariable('user_login_return_login_text') ?? 'Return to login')
         );
 
         $this->output::addResponse($request, $this->render::build([$rowWrapper]));
@@ -238,33 +293,34 @@ class userController extends Controller
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
     public function forgotPasswordCheckEmail(Request $request, Response $response)
     {
         $this->render::setActive("noUI");
-        $this->render::setMetaTitle("Check Email | ". ($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
+        $this->render::setMetaTitle("Check Email | ".($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
 
         $rowWrapper = (new RowWrapper())->addClass('d-block');
         $rowWrapper->addComponent(
             (new ContentTitle())
-            ->addContent("Please check your email for further instructions. The email may take upto a few minutes to arrive.")
-            ->addClass('text-white forgot-password-email-message')
+                ->addContent(
+                    "Please check your email for further instructions. The email may take upto a few minutes to arrive."
+                )
+                ->addClass('text-white')
                 ->setColWidth('md', 12)->setID('forgottenPassword')
         );
         $rowWrapper->addComponent(
-            (new BasicLink())->addClass('forgottenPasswordLink return mt-5 text-decoration-underline')->addRoute(Routes::getRoutePattern('user.display.login'))->addText(System::getVariable('user_login_return_login_text') ?? 'Return to login')
+            (new BasicLink())->addClass('forgottenPasswordLink return mt-5 text-decoration-underline')->addRoute(
+                Routes::getRoutePattern('user.display.login')
+            )->addText(System::getVariable('user_login_return_login_text') ?? 'Return to login')
         );
 
         $this->output::addResponse($request, $this->render::build([$rowWrapper]));
+
         return System::asJson($response);
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
     public function forgotPasswordForm(Request $request, Response $response)
@@ -273,96 +329,16 @@ class userController extends Controller
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
     public function registerForm(Request $request, Response $response)
     {
         $form = $this->primaryAccountForm(true);
+
         return System::asJson($response, $form->getBuiltFormArray());
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
-     * @param array $args
-     * @return Response
-     * @throws \Exception
-     */
-    public function userEditForm(Request $request, Response $response, $args = [])
-    {
-        $form = $this->primaryAccountForm();
-
-        //set form field values
-        $user = new userEntity();
-        if (isset($args[$user->primaryKey])) {
-            $user->load($args[$user->primaryKey]);
-            $form->setMethod('PUT');
-        }
-
-        $form->setActionUrl(Routes::getRoutePattern('userEntity.put'));
-        $form->setSubmitText('Update Now');
-
-        if ($this->currentUser->isAllowed('ChangeUserActiveState')) {
-            $activeField = (new checkboxField('active'))->setLabel('Active (Can Login)');
-            $suspendedField = (new checkboxField('suspended'))->setLabel('Suspended');
-            $invoiceNameField = (new inputField('invoice_name_override'))->setLabel('Invoice Name Override');
-        }
-
-        if ($id = $args[$user->primaryKey] ?? false) {
-            $form->setActionUrl(Routes::getRoutePattern($user->getEntityName().'.put', ['id' => $id]));
-
-            $form->addField($form->getFormField('first_name')->setValue($user->first_name));
-            $form->addField($form->getFormField('last_name')->setValue($user->last_name));
-            $form->addField($form->getFormField('email')->setValue($user->email));
-
-            if ($this->currentUser->isAllowed('ChangeUserActiveState')) {
-                $activeField->setValue($user->active == 1);
-                $suspendedField->setValue($user->suspended == 1);
-                $invoiceNameField->setValue($user->invoice_name_override ?? null);
-            }
-        }
-
-        if ($this->currentUser->isAllowed('ChangeUserActiveState')) {
-            $form->addField($activeField);
-            $form->addField($suspendedField);
-            $form->addField($invoiceNameField);
-        }
-
-        //allow other modules to add form fields.
-        $form = Hook::processHook('displayFormRender', $form, $user, 'userEditForm');
-
-        $roles = (new roleEntity())->loadMultiple();
-        foreach ($roles as $role) {
-            //for users that can't change roles, show labels for those that they do have.
-            $showLabel = false;
-            if ($this->currentUser->isAllowed('UpdateRoles') || $this->currentUser->isAllowed('AssignRole_'.$role->id)) {
-                $roleField = (new checkboxField('role_' . $role->id))->setLabel($role->name);
-            } else {
-                $showLabel = true;
-                $roleField = (new inputField('role_' . $role->id));
-                $roleField->setInputType('hidden');
-            }
-            if (($user->id ?? false) && is_array($user->roles) && ($user && in_array($role->id, $user->roles))) {
-                //allows us to show the label only for those without the role to change.
-                //Only shows labels for the roles they have.
-                if ($showLabel) {
-                    $roleField->setLabel($role->name);
-                }
-                $roleField->setValue($role->id);
-            }
-            $form->addField($roleField, 'roles');
-        }
-
-        //set role values
-        $this->output::addResponse($request, $form->getBuiltFormArray());
-        return System::asJson($response);
-    }
-
-    /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      */
     public function userHome(Request $request, Response $response)
@@ -370,55 +346,29 @@ class userController extends Controller
         $rowWrapper = (new RowWrapper())->setColWidth('sm', 12)->addClass('manageForm-wrapper');
         $rowWrapper->setColWidth('sm', 12)->addClass('manageForm-wrapper');
         $rowWrapper->addComponent(new userEntityDisplayTile());
-        if ($this->currentUser->isAllowed('ViewRoles')) {
+        if ($this->currentUser->isAllowed(Role::VIEW)) {
             $rowWrapper->addComponent(new roleEntityDisplayTile());
         }
 
         $this->render::setPageTitle('User Dashboard');
         $this->output::addResponse($request, $this->render::build([$rowWrapper]));
+
         return System::asJson($response);
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @return Response
      * @throws \Exception
      */
     public function userList(Request $request, Response $response)
     {
-        $buttonBar = new ButtonBar();
-        $buttonBar->setEntity((new userEntity()), 'List');
-
-        $helpComponents = [
-            (new ContentTitle())->addClass('text-black text-start help-modal-content-title')->addTitle('A list of Users, and their details'),
-            (new DisplayTable())->addClass('text-black text-start help-modal-display-table')->addRows([
-                ['ID', 'Unique user reference'],
-                ['Name', 'Customer name'],
-                ['Validated/Active', 'Whether the user is marked as validated. This affects whether a user can log in'],
-                ['Email Address', 'Customer email address'],
-                ['Company', 'Company Name'],
-                ['Last Login', 'Date and Time of Last Login'],
-                ['Created', 'Date this user was created'],
-                ['Roles', 'Data access roles belonging to this user'],
-            ])
-        ];
-
-        $table = (new CardEntityFilteredDisplayTable(new userEntity(), [], true, 'dropmenu'))
-            ->addTitle('Users')
-            ->src(Routes::getRoutePattern('userEntity.get'))
-            ->displayPagination()
-            ->hideIdColumn()
-            ->addProperty('refreshButton', '<i class="fa-light fa-arrows-rotate"></i>')
-        //$table->setFilters('orderBy[]=missing_sims&orderBy[]=DESC&orderBy[]=subscription.id&orderBy[]=DESC');
-            ->addFilterForm('user.display.forms.filters.userFilter', null, 5, true, 'Filter Users')
-            ->addHeaderModal('Users List Table', $helpComponents);
-
-        $table = Hook::processHook('user_list_table_modify', $table);
+        $filterForm = (new EntityFilteredDisplayTable((new userEntity()), [], false))->addEditButton('userEntity.display.put');
+        $filterForm->addFilterForm('user.display.forms.filters.userFilter');
 
         //render
-        $this->render::setPageTitle('Users');
-        $this->output::addResponse($request, $this->render::build([$buttonBar, $table]));
+        $this->render::setPageTitle('User List');
+        $this->output::addResponse($request, $this->render::build($filterForm));
+
         return System::asJson($response);
     }
 
@@ -428,19 +378,20 @@ class userController extends Controller
         if ($id = $args['id'] ?? false) {
             $user = new userEntity();
             $user->load($id);
-            $title = 'Update ' . $user->getNiceName();
+            $title = 'Update '.$user->getNiceName();
         }
 
-        $form = (new FormComponent())->setID('userCreate')->src(Routes::getRoutePattern('user.display.forms.createUpdate', $args));
+        $form = (new FormComponent())->setID('userCreate')->src(
+            Routes::getRoutePattern('user.display.forms.createUpdate', $args)
+        );
         $this->render::setPageTitle($title);
         $this->output::addResponse($request, $this->render::build([genericComponents::dashboardBlockWrapper([$form])]));
+
         return System::asJson($response);
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
-     * @param array $args
+     * @param  array  $args
      * @return Response
      * @throws \Exception
      */
@@ -450,73 +401,79 @@ class userController extends Controller
         if ($id = $args['id'] ?? false) {
             $userEntity = new userEntity();
             $userEntity->load($id);
-            $title = 'Edit ('.$id.') '. $userEntity->getNiceName();
+            $title = 'Edit ('.$id.') '.$userEntity->getNiceName();
         }
 
-        $form = (new FormComponent())->setID('roleCreate')->src(Routes::getRoutePattern('user.display.forms.createUpdate', $args));
+        $form = (new FormComponent())->setID('roleCreate')->src(
+            Routes::getRoutePattern('user.display.forms.createUpdate', $args)
+        );
         $this->render::setPageTitle($title);
         $this->output::addResponse($request, $this->render::build($form));
+
         return System::asJson($response);
     }
 
-    public function suspendUser(Request $request, Response $response, $args = [])
+    public function resetPasswordPage(Request $request, Response $response, $args)
     {
-        $userEntity = new userEntity();
-        $userEntity->load($args['id']);
-        if (!isset($userEntity->id)) {
-            return System::asJson($response, [], 404);
+        $this->render::setActive("noUI");
+        $this->render::setMetaTitle("Forgotten Password | ". ($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
+
+        $hash = $args['hash'] ?? false;
+        $valid = false;
+        $components = [];
+        if ($hash) {
+            $user = new userEntity();
+            $match = $user->getForgottenPasswordLink($hash);
+            if (!empty($match) && ((time() - $match['created']) <= 86400 && $match['used'] == 0)) {
+                $valid = true;
+                $components[] = (new FormComponent())
+                    ->src(Routes::getRoutePattern('user.display.forms.resetPassword', ['hash' => $hash]))
+                    ->title("Reset your password using the fields below:")
+                    ->addClass('forgotten-password-form')->setID('forgottenPassword');
+                $components[] = (new BasicLink())->addClass('forgottenPasswordLink return mt-5 text-decoration-underline')->addRoute(Routes::getRoutePattern('user.display.login'))->addText(System::getVariable('user_login_return_login_text') ?? 'Return to login');
+            }
         }
 
-        $contentTitle = (new ContentTitle())
-            ->addTitle('Are you sure?')
-            ->addContent($userEntity->first_name . ' will no longer be able to log in, and any payment subscriptions they have will be suspended')
-            ;
-        $suspendTile = dashboardTiles::suspendUserTile($userEntity);
-        $components = [$contentTitle, $suspendTile];
+        if (!$valid) {
+            $components[] = (new ContentTitle())->addTitle('Your reset link has expired')->addContent('Please request a new password reset link using the button below:')->addClass('mb-5');
+            $components[] = (new BasicLink())->addClass('btn btn-primary')->addText('Go to Password Reset')->addRoute(Routes::getRoutePattern('user.display.forgot-password'));
+        }
 
-        $components = Hook::processHook('suspendUserPopupContent', $components, $userEntity);
-
-        $rowWrapper = (new RowWrapper())
-            ->addComponent((new ColWrapper())
-                ->addComponents($components)
-            );
-
-        $this->render::setActive("modal");
-        $this->output::addResponse($request, $this->render::build($rowWrapper));
+        $this->output::addResponse($request, $this->render::build($components));
 
         return System::asJson($response);
     }
 
-    /**
-     * @param bool $passwordRequired
-     * @return formEntity
-     */
-    protected function primaryAccountForm($passwordRequired = false): formEntity
+    public function resetPasswordSuccess(Request $request, Response $response, $args)
+    {
+        $this->render::setActive("noUI");
+        $this->render::setMetaTitle("Your password has been reset | ". ($this->config->app->defaultMetaTitle ?? "Io Dashboard"));
+
+        $components = [
+                (new ContentTitle())->addTitle('Password reset successful')->addContent('Please login to the site using your new credentials:')->addClass('mb-5'),
+                (new BasicLink())->addClass('btn btn-primary')->addText('Go to login page')->addRoute(Routes::getRoutePattern('user.display.login'))
+        ];
+
+        $this->output::addResponse($request, $this->render::build($components));
+
+        return System::asJson($response);
+    }
+
+    public function resetPasswordForm(Request $request, Response $response, $args)
     {
         $form = new formEntity();
-        $form->setId('login');
+        $form->setId('reset-password-form');
+        $form->setSubmitText('Reset Password Now');
         $form->setMethod('post');
-        $form->setActionUrl(Routes::getRoutePattern('userEntity.post'));
+        $form->setActionUrl(Routes::getRoutePattern('user.resetPassword'));
+        $form->addField((new inputField('password'))->setLabel('Password')->setPlaceholder('Password')->setInputType('password')->addValidation("minLength", ["min" => 6])->isRequired());
+        $form->addField((new inputField('confirm_password'))->setLabel('Confirm Password')->setPlaceholder('Confirm Password')->setInputType('password')->addValidation("minLength", ["min" => 6])->isRequired());
+        $form->addField((new inputField('hash'))->setInputType('hidden')->setValue($args['hash'])->isRequired());
 
-        $form->addField((new inputField('first_name'))->setLabel('First Name')->setPlaceholder('First Name')->isRequired());
-        $form->addField((new inputField('last_name'))->setLabel('Surname')->setPlaceholder('Surname')->isRequired());
-        $form->addField((new inputField('email'))->setLabel('Email')->setPlaceholder('Email')->setInputType('email')->isRequired());
-
-        $password = (new inputField('password'))->setLabel('Password')->setPlaceholder('Password')->setInputType('password')->addValidation("minLength", ["min" => 6]);
-        $confirmPass = (new inputField('confirm_password'))->setLabel('Confirm Password')->setPlaceholder('Confirm Password')->setInputType('password')->addValidation("minLength", ["min" => 6]);
-        if ($passwordRequired) {
-            $password->isRequired();
-            $confirmPass->isRequired();
-        }
-        $form->addField($password);
-        $form->addField($confirmPass);
-
-        return $form;
+        return System::asJson($response, $form->getBuiltFormArray());
     }
 
     /**
-     * @param Request $request
-     * @param Response $response
      * @param $args
      * @return Response
      */
@@ -529,7 +486,7 @@ class userController extends Controller
                 'Active',
                 'Email',
                 'Last Logged In',
-                'Roles'
+                'Roles',
             ],
             'fields' => [
                 'id',
@@ -537,20 +494,33 @@ class userController extends Controller
                 'active',
                 'email',
                 'last_login',
-                'roles'
+                'roles',
             ],
             'callables' => [
-                'field__last_login' => function ($key, $entityId, $row) : string {
-                    return \apexl\Io\modules\userDisplay\callbacks\tableRowAlters::userShortTable($key, $entityId, $row);
+                'field__last_login' => function ($key, $entityId, $row): string {
+                    return \apexl\Io\modules\userDisplay\callbacks\tableRowAlters::userShortTable(
+                        $key,
+                        $entityId,
+                        $row
+                    );
                 },
-                'field__first_name' => function ($key, $entityId, $row) : string {
-                    return \apexl\Io\modules\userDisplay\callbacks\tableRowAlters::userShortTable($key, $entityId, $row);
+                'field__first_name' => function ($key, $entityId, $row): string {
+                    return \apexl\Io\modules\userDisplay\callbacks\tableRowAlters::userShortTable(
+                        $key,
+                        $entityId,
+                        $row
+                    );
                 },
-                'field__roles' => function ($key, $entityId, $row) : string {
-                    return \apexl\Io\modules\userDisplay\callbacks\tableRowAlters::userShortTable($key, $entityId, $row);
-                }
-            ]
+                'field__roles' => function ($key, $entityId, $row): string {
+                    return \apexl\Io\modules\userDisplay\callbacks\tableRowAlters::userShortTable(
+                        $key,
+                        $entityId,
+                        $row
+                    );
+                },
+            ],
         ];
+
         return $this->getEntityData($request, $response, $args, (new userEntity()), [], $structure);
     }
 }
